@@ -11,6 +11,7 @@ import libcst
 from pytype import __version__
 from pytype import analyze
 from pytype import config
+from pytype import constant_folding
 from pytype import directors
 from pytype import errors
 from pytype import load_pytd
@@ -64,14 +65,12 @@ def _set_verbosity_from(posarg):
 @_set_verbosity_from(posarg=2)
 def _call(analyze_types, src, options, loader):
   """Helper function to call analyze.check/infer_types."""
-  errorlog = errors.ErrorLog()
   # 'deep' tells the analyzer whether to analyze functions not called from main.
   deep = not options.main_only
   loader = loader or load_pytd.create_loader(options)
-  return errorlog, analyze_types(
+  return analyze_types(
       src=src,
       filename=options.input,
-      errorlog=errorlog,
       options=options,
       loader=loader,
       deep=deep)
@@ -81,8 +80,8 @@ def check_py(src, options=None, loader=None):
   """Check the types of a string of source code."""
   options = options or config.Options.create()
   with config.verbosity_from(options):
-    errorlog, _ = _call(analyze.check_types, src, options, loader)
-  return errorlog
+    ret = _call(analyze.check_types, src, options, loader)
+  return ret.errorlog
 
 
 def generate_pyi(src, options=None, loader=None):
@@ -102,10 +101,11 @@ def generate_pyi(src, options=None, loader=None):
   """
   options = options or config.Options.create()
   with config.verbosity_from(options):
-    errorlog, (mod, builtins) = _call(analyze.infer_types, src, options, loader)
+    ret = _call(analyze.infer_types, src, options, loader)
+    mod = ret.ast
     mod.Visit(visitors.VerifyVisitor())
     mod = optimize.Optimize(mod,
-                            builtins,
+                            ret.builtins,
                             lossy=False,
                             use_abcs=False,
                             max_union=7,
@@ -119,7 +119,7 @@ def generate_pyi(src, options=None, loader=None):
   result += "\n"
   if options.quick:
     result = "# (generated with --quick)\n\n" + result
-  return errorlog, result, mod
+  return ret.errorlog, result, mod
 
 
 @_set_verbosity_from(posarg=0)
@@ -149,6 +149,8 @@ def check_or_generate_pyi(options, loader=None):
     raise
   except pyc.CompileError as e:
     errorlog.python_compiler_error(options.input, e.lineno, e.error)
+  except constant_folding.ConstantError as e:
+    errorlog.python_compiler_error(options.input, e.lineno, e.message)
   except IndentationError as e:
     errorlog.python_compiler_error(options.input, e.lineno, e.msg)
   except libcst.ParserSyntaxError as e:
