@@ -1,8 +1,11 @@
 """Implementation of named tuples."""
 
+import dataclasses
 from keyword import iskeyword  # pylint: disable=g-importing-member
 import textwrap
 
+from typing import Any, List
+
 from pytype import overlay_utils
 from pytype import utils
 from pytype.abstract import abstract
@@ -19,6 +22,50 @@ from pytype.pytd import visitors
 Param = overlay_utils.Param
 
 
+@dataclasses.dataclass
+class Field:
+  """A namedtuple field."""
+
+  name: str
+  typ: Any
+  default: Any = None
+
+
+@dataclasses.dataclass
+class NamedTupleProperties:
+  """Collection of properties used to construct a namedtuple."""
+
+  name: str
+  fields: List[Field]
+  bases: List[Any]
+
+  def validate_and_rename_fields(self, rename):
+    """Validate and rename self.fields."""
+    # namedtuple field names have some requirements:
+    # - must not be a Python keyword
+    # - must consist of only alphanumeric characters and "_"
+    # - must not start with "_" or a digit
+    # Basically, they're valid Python identifiers that don't start with "_" or a
+    # digit. Also, there can be no duplicate field names.
+    # Typename has the same requirements, except it can start with "_".
+    # If rename is true, any invalid field names are changed to "_%d". For
+    # example, "abc def ghi abc" becomes "abc _1 def _3" because "def" is a
+    # keyword and "abc" is a duplicate.
+    # The typename cannot be changed.
+
+    if _invalid_name(self.name):
+      raise ValueError(self.name)
+
+    seen = set()
+    for idx, f in enumerate(self.fields):
+      if _invalid_name(f.name) or f.name.startswith("_") or f.name in seen:
+        if rename:
+          f.name = "_%d" % idx
+        else:
+          raise ValueError(f.name)
+      seen.add(f.name)
+
+
 def _repeat_type(type_str, n):
   return ", ".join((type_str,) * n) if n else "()"
 
@@ -194,32 +241,15 @@ class NamedTupleBuilder(abstract.PyTDFunction):
     return name_var, field_names, defaults, rename
 
   def _validate_and_rename_args(self, typename, field_names, rename):
-    # namedtuple field names have some requirements:
-    # - must not be a Python keyword
-    # - must consist of only alphanumeric characters and "_"
-    # - must not start with "_" or a digit
-    # Basically, they're valid Python identifiers that don't start with "_" or a
-    # digit. Also, there can be no duplicate field names.
-    # Typename has the same requirements, except it can start with "_".
-    # If rename is true, any invalid field names are changed to "_%d". For
-    # example, "abc def ghi abc" becomes "abc _1 def _3" because "def" is a
-    # keyword and "abc" is a duplicate.
-    # The typename cannot be changed.
+    # See comment in NamedTupleProperties.validate_and_rename_fields
 
-    # Small helper function for checking typename and field names.
-    def not_valid(field_name):
-      return (not all(c.isalnum() or c == "_" for c in field_name)
-              or iskeyword(field_name)
-              or not field_name  # catches empty string, etc.
-              or field_name[0].isdigit())
-
-    if not_valid(typename):
+    if _invalid_name(typename):
       raise ValueError(typename)
 
     valid_fields = list(field_names)
     seen = set()
     for idx, name in enumerate(field_names):
-      if not_valid(name) or name.startswith("_") or name in seen:
+      if _invalid_name(name) or name.startswith("_") or name in seen:
         if rename:
           valid_fields[idx] = "_%d" % idx
         else:
@@ -334,7 +364,7 @@ class NamedTupleFuncBuilder(NamedTupleBuilder):
     return (isinstance(val, abstract.Instance) and
             val.full_name in ("builtins.str", "builtins.unicode"))
 
-  def _getargs(self, node, args, functional):
+  def _getargs(self, node, args):
     self.match_args(node, args)
     sig, = self.signatures
     callargs = {name: var for name, var, _ in sig.signature.iter_args(args)}
@@ -366,200 +396,20 @@ class NamedTupleFuncBuilder(NamedTupleBuilder):
       name, typ = field
       name_py_constant = abstract_utils.get_atomic_python_constant(name)
       names.append(name_py_constant)
-      if functional:
-        allowed_type_params = (
-            self.ctx.annotation_utils.get_callable_type_parameter_names(typ))
-        annot = self.ctx.annotation_utils.extract_annotation(
-            node,
-            typ,
-            name_py_constant,
-            self.ctx.vm.simple_stack(),
-            allowed_type_params=allowed_type_params)
-      else:
-        # This NamedTuple was constructed with the class syntax. The field
-        # annotations were already processed when added to __annotations__.
-        annot = abstract_utils.get_atomic_value(typ)
+      allowed_type_params = (
+          self.ctx.annotation_utils.get_callable_type_parameter_names(typ))
+      annot = self.ctx.annotation_utils.extract_annotation(
+          node,
+          typ,
+          name_py_constant,
+          self.ctx.vm.simple_stack(),
+          allowed_type_params=allowed_type_params)
       types.append(annot)
     return name_var, names, types
 
-  def _build_namedtuple(self, name, field_names, field_types, node, bases):
-    # Build an InterpreterClass representing the namedtuple.
-    if field_types:
-      # TODO(mdemello): Fix this to support late types.
-      field_types_union = abstract.Union(field_types, self.ctx)
-    else:
-      field_types_union = self.ctx.convert.none_type
-
-    members = {n: t.instantiate(node) for n, t in zip(field_names, field_types)}
-
-    # collections.namedtuple has: __dict__, __slots__ and _fields.
-    # typing.NamedTuple adds: _field_types, __annotations__ and _field_defaults.
-    # __slots__ and _fields are tuples containing the names of the fields.
-    slots = tuple(self.ctx.convert.build_string(node, f) for f in field_names)
-    members["__slots__"] = self.ctx.convert.build_tuple(node, slots)
-    members["_fields"] = self.ctx.convert.build_tuple(node, slots)
-    # __dict__ and _field_defaults are both collections.OrderedDicts that map
-    # field names (strings) to objects of the field types.
-    ordered_dict_cls = self.ctx.convert.name_to_value(
-        "collections.OrderedDict", ast=self.collections_ast)
-
-    # Normally, we would use abstract_utils.K and abstract_utils.V, but
-    # collections.pyi doesn't conform to that standard.
-    field_dict_cls = abstract.ParameterizedClass(ordered_dict_cls, {
-        "K": self.ctx.convert.str_type,
-        "V": field_types_union
-    }, self.ctx)
-    members["__dict__"] = field_dict_cls.instantiate(node)
-    members["_field_defaults"] = field_dict_cls.instantiate(node)
-    # _field_types and __annotations__ are both collections.OrderedDicts
-    # that map field names (strings) to the types of the fields. Note that
-    # ctx.make_class will take care of adding the __annotations__ member.
-    field_types_cls = abstract.ParameterizedClass(ordered_dict_cls, {
-        "K": self.ctx.convert.str_type,
-        "V": self.ctx.convert.type_type
-    }, self.ctx)
-    members["_field_types"] = field_types_cls.instantiate(node)
-
-    # __new__
-    # We set the bound on this TypeParameter later. This gives __new__ the
-    # signature: def __new__(cls: Type[_Tname], ...) -> _Tname, i.e. the same
-    # signature that visitor.CreateTypeParametersForSignatures would create.
-    # This allows subclasses of the NamedTuple to get the correct type from
-    # their constructors.
-    cls_type_param = abstract.TypeParameter(
-        visitors.CreateTypeParametersForSignatures.PREFIX + name,
-        self.ctx,
-        bound=None)
-    cls_type = abstract.ParameterizedClass(self.ctx.convert.type_type,
-                                           {abstract_utils.T: cls_type_param},
-                                           self.ctx)
-    params = [Param(n, t) for n, t in zip(field_names, field_types)]
-    members["__new__"] = overlay_utils.make_method(
-        self.ctx,
-        node,
-        name="__new__",
-        self_param=Param("cls", cls_type),
-        params=params,
-        return_type=cls_type_param,
-    )
-
-    # __init__
-    members["__init__"] = overlay_utils.make_method(
-        self.ctx,
-        node,
-        name="__init__",
-        varargs=Param("args"),
-        kwargs=Param("kwargs"))
-
-    heterogeneous_tuple_type_params = dict(enumerate(field_types))
-    heterogeneous_tuple_type_params[abstract_utils.T] = field_types_union
-    # Representation of the to-be-created NamedTuple as a typing.Tuple.
-    heterogeneous_tuple_type = abstract.TupleClass(
-        self.ctx.convert.tuple_type, heterogeneous_tuple_type_params, self.ctx)
-
-    # _make
-    # _make is a classmethod, so it needs to be wrapped by
-    # special_builtins.ClassMethodInstance.
-    # Like __new__, it uses the _Tname TypeVar.
-    sized_cls = self.ctx.convert.name_to_value("typing.Sized")
-    iterable_type = abstract.ParameterizedClass(
-        self.ctx.convert.name_to_value("typing.Iterable"),
-        {abstract_utils.T: field_types_union}, self.ctx)
-    cls_type = abstract.ParameterizedClass(self.ctx.convert.type_type,
-                                           {abstract_utils.T: cls_type_param},
-                                           self.ctx)
-    len_type = abstract.CallableClass(
-        self.ctx.convert.name_to_value("typing.Callable"), {
-            0: sized_cls,
-            abstract_utils.ARGS: sized_cls,
-            abstract_utils.RET: self.ctx.convert.int_type
-        }, self.ctx)
-    params = [
-        Param("iterable", iterable_type),
-        Param("new").unsolvable(self.ctx, node),
-        Param("len", len_type).unsolvable(self.ctx, node)
-    ]
-    make = overlay_utils.make_method(
-        self.ctx,
-        node,
-        name="_make",
-        params=params,
-        self_param=Param("cls", cls_type),
-        return_type=cls_type_param)
-    make_args = function.Args(posargs=(make,))
-    _, members["_make"] = self.ctx.special_builtins["classmethod"].call(
-        node, None, make_args)
-
-    # _replace
-    # Like __new__, it uses the _Tname TypeVar. We have to annotate the `self`
-    # param to make sure the TypeVar is substituted correctly.
-    members["_replace"] = overlay_utils.make_method(
-        self.ctx,
-        node,
-        name="_replace",
-        self_param=Param("self", cls_type_param),
-        return_type=cls_type_param,
-        kwargs=Param("kwds", field_types_union))
-
-    # __getnewargs__
-    members["__getnewargs__"] = overlay_utils.make_method(
-        self.ctx,
-        node,
-        name="__getnewargs__",
-        return_type=heterogeneous_tuple_type)
-
-    # __getstate__
-    members["__getstate__"] = overlay_utils.make_method(
-        self.ctx, node, name="__getstate__")
-
-    # _asdict
-    members["_asdict"] = overlay_utils.make_method(
-        self.ctx, node, name="_asdict", return_type=field_dict_cls)
-
-    # Finally, make the class.
-    cls_dict = abstract.Dict(self.ctx)
-    cls_dict.update(node, members)
-
-    if self.ctx.options.strict_namedtuple_checks:
-      # Enforces type checking like Tuple[...]
-      superclass_of_new_type = heterogeneous_tuple_type.to_variable(node)
-    else:
-      superclass_of_new_type = self.ctx.convert.tuple_type.to_variable(node)
-    if bases:
-      final_bases = []
-      for base in bases:
-        if any(b.full_name == "typing.NamedTuple" for b in base.data):
-          final_bases.append(superclass_of_new_type)
-        else:
-          final_bases.append(base)
-    else:
-      final_bases = [superclass_of_new_type]
-      # This NamedTuple is being created via a function call. We manually
-      # construct an annotated_locals entry for it so that __annotations__ is
-      # initialized properly for the generated class.
-      self.ctx.vm.annotated_locals[name] = {
-          field: abstract_utils.Local(node, None, typ, None, self.ctx)
-          for field, typ in zip(field_names, field_types)
-      }
-
-    node, cls_var = self.ctx.make_class(
-        node=node,
-        name_var=self.ctx.convert.build_string(node, name),
-        bases=final_bases,
-        class_dict_var=cls_dict.to_variable(node),
-        cls_var=None)
-    cls = cls_var.data[0]
-
-    # Now that the class has been made, we can complete the TypeParameter used
-    # by __new__, _make and _replace.
-    cls_type_param.bound = cls
-
-    return node, cls_var
-
   def call(self, node, _, args, bases=None):
     try:
-      name_var, field_names, field_types = self._getargs(
-          node, args, functional=bases is None)
+      name_var, field_names, field_types = self._getargs(node, args)
     except abstract_utils.ConversionError:
       return node, self.ctx.new_unsolvable(node)
 
@@ -581,13 +431,32 @@ class NamedTupleFuncBuilder(NamedTupleBuilder):
         annots.get(field_name, self.ctx.convert.unsolvable)
         for field_name in field_names
     ]
-    node, cls_var = self._build_namedtuple(
-        name, field_names, field_types, node, bases)
+    fields = [Field(n, t) for n, t in zip(field_names, field_types)]
+    props = NamedTupleProperties(name, fields, bases)
+    node, cls_var = _build_namedtuple(props, node, self.ctx)
 
     self.ctx.vm.trace_classdef(cls_var)
     return node, cls_var
 
 
+class NamedTupleClass(abstract.PyTDClass):
+  """Representation of NamedTuple classes."""
+
+  def __init__(self, props, ctx):
+    self.props = props
+    typing_ast = ctx.loader.import_name("typing")
+    # Because NamedTuple is a special case for the pyi parser, typing.pytd has
+    # "_NamedTuple" instead. Replace the name of the returned function so that
+    # error messages will correctly display "typing.NamedTuple".
+    pyval = typing_ast.Lookup("typing._NamedTuple")
+    pyval = pyval.Replace(name="typing.NamedTuple")
+    super().__init__(props.name, pyval, ctx)
+    self.init_method = self._make_init(props)
+
+  def __repr__(self):
+    return f"NamedTupleClass({self.name})"
+
+
 class NamedTupleClassBuilder(abstract.PyTDClass):
   """Factory for creating typing.NamedTuple classes."""
 
@@ -641,39 +510,44 @@ class NamedTupleClassBuilder(abstract.PyTDClass):
 
     # retrieve __qualname__ to get the name of class
     name = f_locals["__qualname__"]
-    nameval = abstract_utils.get_atomic_python_constant(name)
-    if "." in nameval:
-      nameval = nameval.rsplit(".", 1)[-1]
-      name = self.ctx.convert.constant_to_var(nameval)
+    name = abstract_utils.get_atomic_python_constant(name)
+    if "." in name:
+      name = name.rsplit(".", 1)[-1]
 
-    # assemble the arguments that are compatible with NamedTupleFuncBuilder.call
-    field_list = []
-    defaults = []
+    # Construct a NamedTupleProperties to pass to the builder function.
     cls_locals = classgen.get_class_locals(
-        nameval,
+        name,
         allow_methods=True,
         ordering=classgen.Ordering.FIRST_ANNOTATE,
         ctx=self.ctx)
+    props = NamedTupleProperties(name=name, fields=[], bases=bases)
+    stack = tuple(self.ctx.vm.frames)
     for k, local in cls_locals.items():
       assert local.typ
-      if k in f_locals:
-        defaults.append(f_locals[k])
-      k = self.ctx.convert.constant_to_var(k, node=node)
-      field_list.append(self.ctx.convert.build_tuple(node, (k, local.typ)))
-    anno = self.ctx.convert.build_list(node, field_list)
-    posargs = (name, anno)
-    args = function.Args(posargs=posargs)
-    node, cls_var = self.namedtuple.call(node, None, args, bases)
+      t = self.ctx.annotation_utils.extract_annotation(
+          node, local.typ, k, stack)
+      props.fields.append(Field(name=k, typ=t, default=f_locals.get(k)))
+
+    # typing.NamedTuple doesn't support rename; invalid fields are an error.
+    try:
+      props.validate_and_rename_fields(rename=False)
+    except ValueError as e:
+      self.ctx.errorlog.invalid_namedtuple_arg(
+          self.ctx.vm.frames, utils.message(e))
+      return node, self.ctx.new_unsolvable(node)
+
+    node, cls_var = _build_namedtuple(props, node, self.ctx)
     cls_val = abstract_utils.get_atomic_value(cls_var)
 
     if not isinstance(cls_val, abstract.Unsolvable):
       # set __new__.__defaults__
+      defaults = [f.default for f in props.fields if f.default is not None]
       defaults = self.ctx.convert.build_tuple(node, defaults)
       node, new_attr = self.ctx.attribute_handler.get_attribute(
           node, cls_val, "__new__")
       new_attr = abstract_utils.get_atomic_value(new_attr)
-      node = self.ctx.attribute_handler.set_attribute(node, new_attr,
-                                                      "__defaults__", defaults)
+      node = self.ctx.attribute_handler.set_attribute(
+          node, new_attr, "__defaults__", defaults)
 
       # set the attribute without overriding special namedtuple attributes
       node, fields = self.ctx.attribute_handler.get_attribute(
@@ -689,3 +563,199 @@ class NamedTupleClassBuilder(abstract.PyTDClass):
               node, cls_val, key, f_locals[key])
 
     return node, cls_var
+
+
+# Small helper function for checking typename and field names.
+def _invalid_name(field_name):
+  return (not all(c.isalnum() or c == "_" for c in field_name)
+          or iskeyword(field_name)
+          or not field_name  # catches empty string, etc.
+          or field_name[0].isdigit())
+
+
+class _DictBuilder:
+  """Construct OrderedDict abstract classes for namedtuple members."""
+
+  def __init__(self, ctx):
+    self.ctx = ctx
+    collections_ast = ctx.loader.import_name("collections")
+    self.ordered_dict_cls = ctx.convert.name_to_value(
+        "collections.OrderedDict", ast=collections_ast)
+
+  def make(self, typ):
+    # Normally, we would use abstract_utils.K and abstract_utils.V, but
+    # collections.pyi doesn't conform to that standard.
+    return abstract.ParameterizedClass(self.ordered_dict_cls, {
+        "K": self.ctx.convert.str_type,
+        "V": typ
+    }, self.ctx)
+
+
+def _build_namedtuple(props, node, ctx):
+  """Build an InterpreterClass representing the namedtuple."""
+
+  # TODO(mdemello): Fix this to support late types.
+  if props.fields and props.fields[0].typ:
+    field_types_union = abstract.Union([f.typ for f in props.fields], ctx)
+  else:
+    field_types_union = ctx.convert.none_type
+
+  members = {f.name: f.typ.instantiate(node) for f in props.fields}
+
+  # collections.namedtuple has: __dict__, __slots__ and _fields.
+  # typing.NamedTuple adds: _field_types, __annotations__ and _field_defaults.
+  # __slots__ and _fields are tuples containing the names of the fields.
+  slots = tuple(ctx.convert.build_string(node, f.name) for f in props.fields)
+  members["__slots__"] = ctx.convert.build_tuple(node, slots)
+  members["_fields"] = ctx.convert.build_tuple(node, slots)
+
+  odict = _DictBuilder(ctx)
+  # __dict__ and _field_defaults are both OrderedDicts of
+  # { field_name: field_type_instance }
+  field_dict_cls = odict.make(field_types_union)
+  members["__dict__"] = field_dict_cls.instantiate(node)
+  members["_field_defaults"] = field_dict_cls.instantiate(node)
+
+  # _field_types and __annotations__ are both OrderedDicts of
+  # { field_name: field_type }
+  # Note that ctx.make_class will take care of adding the __annotations__
+  # member.
+  field_types_cls = odict.make(ctx.convert.type_type)
+  members["_field_types"] = field_types_cls.instantiate(node)
+
+  # __new__
+  # We set the bound on this TypeParameter later. This gives __new__ the
+  # signature: def __new__(cls: Type[_Tname], ...) -> _Tname, i.e. the same
+  # signature that visitor.CreateTypeParametersForSignatures would create.
+  # This allows subclasses of the NamedTuple to get the correct type from
+  # their constructors.
+  cls_type_param = abstract.TypeParameter(
+      visitors.CreateTypeParametersForSignatures.PREFIX + props.name,
+      ctx,
+      bound=None)
+  cls_type = abstract.ParameterizedClass(ctx.convert.type_type,
+                                         {abstract_utils.T: cls_type_param},
+                                         ctx)
+
+  params = [Param(f.name, f.typ) for f in props.fields]
+  members["__new__"] = overlay_utils.make_method(
+      ctx,
+      node,
+      name="__new__",
+      self_param=Param("cls", cls_type),
+      params=params,
+      return_type=cls_type_param,
+  )
+
+  # __init__
+  members["__init__"] = overlay_utils.make_method(
+      ctx,
+      node,
+      name="__init__",
+      varargs=Param("args"),
+      kwargs=Param("kwargs"))
+
+  heterogeneous_tuple_type_params = dict(enumerate(f.typ for f in props.fields))
+  heterogeneous_tuple_type_params[abstract_utils.T] = field_types_union
+  # Representation of the to-be-created NamedTuple as a typing.Tuple.
+  heterogeneous_tuple_type = abstract.TupleClass(
+      ctx.convert.tuple_type, heterogeneous_tuple_type_params, ctx)
+
+  # _make
+  # _make is a classmethod, so it needs to be wrapped by
+  # special_builtins.ClassMethodInstance.
+  # Like __new__, it uses the _Tname TypeVar.
+  sized_cls = ctx.convert.name_to_value("typing.Sized")
+  iterable_type = abstract.ParameterizedClass(
+      ctx.convert.name_to_value("typing.Iterable"),
+      {abstract_utils.T: field_types_union}, ctx)
+  cls_type = abstract.ParameterizedClass(ctx.convert.type_type,
+                                         {abstract_utils.T: cls_type_param},
+                                         ctx)
+  len_type = abstract.CallableClass(
+      ctx.convert.name_to_value("typing.Callable"), {
+          0: sized_cls,
+          abstract_utils.ARGS: sized_cls,
+          abstract_utils.RET: ctx.convert.int_type
+      }, ctx)
+  params = [
+      Param("iterable", iterable_type),
+      Param("new").unsolvable(ctx, node),
+      Param("len", len_type).unsolvable(ctx, node)
+  ]
+  make = overlay_utils.make_method(
+      ctx,
+      node,
+      name="_make",
+      params=params,
+      self_param=Param("cls", cls_type),
+      return_type=cls_type_param)
+  make_args = function.Args(posargs=(make,))
+  _, members["_make"] = ctx.special_builtins["classmethod"].call(
+      node, None, make_args)
+
+  # _replace
+  # Like __new__, it uses the _Tname TypeVar. We have to annotate the `self`
+  # param to make sure the TypeVar is substituted correctly.
+  members["_replace"] = overlay_utils.make_method(
+      ctx,
+      node,
+      name="_replace",
+      self_param=Param("self", cls_type_param),
+      return_type=cls_type_param,
+      kwargs=Param("kwds", field_types_union))
+
+  # __getnewargs__
+  members["__getnewargs__"] = overlay_utils.make_method(
+      ctx,
+      node,
+      name="__getnewargs__",
+      return_type=heterogeneous_tuple_type)
+
+  # __getstate__
+  members["__getstate__"] = overlay_utils.make_method(
+      ctx, node, name="__getstate__")
+
+  # _asdict
+  members["_asdict"] = overlay_utils.make_method(
+      ctx, node, name="_asdict", return_type=field_dict_cls)
+
+  # Finally, make the class.
+  cls_dict = abstract.Dict(ctx)
+  cls_dict.update(node, members)
+
+  if ctx.options.strict_namedtuple_checks:
+    # Enforces type checking like Tuple[...]
+    superclass_of_new_type = heterogeneous_tuple_type.to_variable(node)
+  else:
+    superclass_of_new_type = ctx.convert.tuple_type.to_variable(node)
+  if props.bases:
+    final_bases = []
+    for base in props.bases:
+      if any(b.full_name == "typing.NamedTuple" for b in base.data):
+        final_bases.append(superclass_of_new_type)
+      else:
+        final_bases.append(base)
+  else:
+    final_bases = [superclass_of_new_type]
+    # This NamedTuple is being created via a function call. We manually
+    # construct an annotated_locals entry for it so that __annotations__ is
+    # initialized properly for the generated class.
+    ctx.vm.annotated_locals[props.name] = {
+        f.name: abstract_utils.Local(node, None, f.typ, None, ctx)
+        for f in props.fields
+    }
+
+  node, cls_var = ctx.make_class(
+      node=node,
+      name_var=ctx.convert.build_string(node, props.name),
+      bases=final_bases,
+      class_dict_var=cls_dict.to_variable(node),
+      cls_var=None)
+  cls = cls_var.data[0]
+
+  # Now that the class has been made, we can complete the TypeParameter used
+  # by __new__, _make and _replace.
+  cls_type_param.bound = cls
+
+  return node, cls_var
