@@ -16,6 +16,7 @@ class TypingExtensionsOverlay(overlay.Overlay):
         "Protocol": _build("typing.Protocol"),
         "runtime": _build("typing.runtime_checkable"),
         "SupportsIndex": _build("typing_extensions.SupportsIndex", ast),
+        "TypeAlias": _build("typing.TypeAlias"),
         "TypedDict": typing_overlay.typing_overlay["TypedDict"],
     }
     for pyval in ast.aliases + ast.classes + ast.constants + ast.functions:
@@ -26,8 +27,7 @@ class TypingExtensionsOverlay(overlay.Overlay):
         continue
       if f"typing.{name}" not in ctx.loader.typing:
         if name not in member_map:
-          member_map[name] = overlay.build(
-              name, typing_overlay.not_supported_yet)
+          member_map[name] = _build_not_supported_yet(name, ast)
     super().__init__(ctx, "typing_extensions", member_map, ast)
 
   def _convert_member(self, member, subst=None):
@@ -43,3 +43,7 @@ class TypingExtensionsOverlay(overlay.Overlay):
 
 def _build(name, ast=None):
   return lambda ctx: ctx.convert.name_to_value(name, ast=ast)
+
+
+def _build_not_supported_yet(name, ast):
+  return lambda ctx: typing_overlay.not_supported_yet(name, ctx, ast)
