@@ -40,6 +40,10 @@ _TUPLE_TYPES = ("tuple", "builtins.tuple", "typing.Tuple")
 _TYPEGUARD_TYPES = ("typing.TypeGuard", "typing_extensions.TypeGuard")
 
 
+class StringParseError(ParseError):
+  pass
+
+
 def _split_definitions(defs: List[Any]):
   """Return [constants], [functions] given a mixed list of definitions."""
   constants = []
@@ -139,15 +143,23 @@ def _maybe_resolve_alias(alias, name_to_class, name_to_constant):
     return value.Replace(name=alias.name)
 
 
-def pytd_literal(parameters: List[Any]) -> pytd.Type:
+def pytd_literal(
+    parameters: List[Any], aliases: Dict[str, pytd.Alias]) -> pytd.Type:
   """Create a pytd.Literal."""
   literal_parameters = []
   for p in parameters:
     if pytdgen.is_none(p):
       literal_parameters.append(p)
     elif isinstance(p, pytd.NamedType):
-      # TODO(b/173742489): support enums.
-      literal_parameters.append(pytd.AnythingType())
+      prefix = p.name.rsplit(".", 1)[0]
+      # If prefix is a module name, then p is an alias to a Literal in another
+      # module. Otherwise, prefix is an enum type and p is a member of the enum.
+      if prefix in aliases and isinstance(aliases[prefix].type, pytd.Module):
+        literal_parameters.append(p)
+      else:
+        literal_parameters.append(pytd.Literal(
+            pytd.Constant(name=p.name, type=pytd.NamedType(prefix))
+        ))
     elif isinstance(p, types.Pyval):
       literal_parameters.append(p.to_pytd_literal())
     elif isinstance(p, pytd.Literal):
@@ -493,7 +505,7 @@ class Definitions:
   def _parameterized_type(self, base_type: Any, parameters):
     """Return a parameterized type."""
     if self._matches_named_type(base_type, _LITERAL_TYPES):
-      return pytd_literal(parameters)
+      return pytd_literal(parameters, self.aliases)
     elif self._matches_named_type(base_type, _ANNOTATED_TYPES):
       return pytd_annotated(parameters)
     elif self._matches_named_type(base_type, _FINAL_TYPES):
@@ -504,10 +516,15 @@ class Definitions:
       # blocking typeshed, convert type guards to plain bools.
       return pytd.NamedType("bool")
     elif any(isinstance(p, types.Pyval) for p in parameters):
+      if all(not isinstance(p, types.Pyval) or
+             p.type == "str" and p.value for p in parameters):
+        error_cls = StringParseError
+      else:
+        error_cls = ParseError
       parameters = ", ".join(
           p.repr_str() if isinstance(p, types.Pyval) else "_"
           for p in parameters)
-      raise ParseError(
+      raise error_cls(
           "%s[%s] not supported" % (pytd_utils.Print(base_type), parameters))
     elif pytdgen.is_any(base_type):
       return pytd.AnythingType()
